@@ -2,8 +2,9 @@
 #include <fcntl.h>
 #include <string.h>
 #include <errno.h>
-#include <linux/zaptel.h>
+#include "zaptel.h"
 #include <stdio.h>
+#include <linux/types.h>
 #include <linux/ppp_defs.h> 
 #include <sys/ioctl.h>
 #include <unistd.h>
