@@ -65,6 +65,11 @@ static void echo_can_init(void)
 	hpec_init(logger, debug, ZT_CHUNKSIZE, memalloc, memfree);
 }
 
+static void echo_can_identify(char *buf, size_t len)
+{
+	strncpy(buf, "HPEC", len);
+}
+
 static void echo_can_shutdown(void)
 {
 	hpec_shutdown();
@@ -98,7 +103,7 @@ static inline struct echo_can_state *echo_can_create(int len, int adaption_mode)
 
 static inline int echo_can_traintap(struct echo_can_state *ec, int pos, short val)
 {
-	return 1;
+	return 0;
 }
 
 DECLARE_MUTEX(license_lock);
