@@ -146,6 +146,11 @@ static void echo_can_init(void)
 	printk("Zaptel Echo Canceller: KB1%s\n", ZAPTEL_ECHO_AGGRESSIVE);
 }
 
+static void echo_can_identify(char *buf, size_t len)
+{
+	strncpy(buf, "KB1", len);
+}
+
 static void echo_can_shutdown(void)
 {
 }
