@@ -103,6 +103,11 @@ static void echo_can_init(void)
 	printk("Zaptel Echo Canceller: MARK%s\n", ZAPTEL_ECHO_AGGRESSIVE);
 }
 
+static void echo_can_identify(char *buf, size_t len)
+{
+	strncpy(buf, "MARK2", len);
+}
+
 static void echo_can_shutdown(void)
 {
 }
