@@ -63,7 +63,7 @@ typedef struct {
 	int maxexp;
 } cbuf_f;
 
-typedef struct {
+struct echo_can_state {
 	float  	a[NTAPS];				/* Coefficients */
 #ifdef COEFF_BACKUP
 	float  	b[NTAPS];				/* Coefficients */
@@ -77,9 +77,9 @@ typedef struct {
 	int		taps;					/* Number of taps */
 	int		hcntr;					/* Hangtime counter */
 	int pos;						/* Position in curcular buffers */
-} echo_can_state_t;
+};
 
-static inline void echo_can_free(echo_can_state_t *ec)
+static inline void echo_can_free(struct echo_can_state *ec)
 {
 	FREE(ec);
 }
@@ -106,7 +106,7 @@ static inline void buf_add(cbuf_f *b, float sample, int pos, int taps)
 	}
 }
 
-static inline short echo_can_update(echo_can_state_t *ec, short iref, short isig)
+static inline short echo_can_update(struct echo_can_state *ec, short iref, short isig)
 {
 	int x;
 	float ref;
@@ -210,14 +210,14 @@ static inline short echo_can_update(echo_can_state_t *ec, short iref, short isig
 	return (short)(u);
 }
 
-static inline echo_can_state_t *echo_can_create(int taps, int adaption_mode)
+static inline struct echo_can_state *echo_can_create(int taps, int adaption_mode)
 {
-	echo_can_state_t *ec;
+	struct echo_can_state *ec;
 	taps = NTAPS;
-	ec = MALLOC(sizeof(echo_can_state_t));
+	ec = MALLOC(sizeof(struct echo_can_state));
 	if (ec) {
 		printk("Allocating MEC3 canceller (%d)\n", taps);
-		memset(ec, 0, sizeof(echo_can_state_t));
+		memset(ec, 0, sizeof(struct echo_can_state));
 		ec->taps = taps;
 		if (ec->taps > NTAPS)
 			ec->taps = NTAPS;
