@@ -41,20 +41,35 @@
 #define FREE(a) free(a)
 #endif
 
+
 #define ABS(a) abs(a!=-32768?a:-32767)
 
+#define RESTORE_COEFFS {\
+				int x;\
+				memcpy(ec->a_i, ec->c_i, ec->N_d*sizeof(int));\
+				for (x=0;x<ec->N_d;x++) {\
+					ec->a_s[x] = ec->a_i[x] >> 16;\
+				}\
+				ec->backup = BACKUP;\
+			}
+
 /* Uncomment to provide summary statistics for overall echo can performance every 4000 samples */ 
 /* #define MEC2_STATS 4000 */
 
 /* Uncomment to generate per-sample statistics - this will severely degrade system performance and audio quality */
 /* #define MEC2_STATS_DETAILED */
 
+/* Uncomment to generate per-call DC bias offset messages */
+/* #define MEC2_DCBIAS_MESSAGE */
+
 /* Get optimized routines for math */
 #include "arith.h"
 
 /* Bring in definitions for the various constants and thresholds */
 #include "mg2ec_const.h"
 
+#define DC_NORMALIZE
+
 #ifndef NULL
 #define NULL 0
 #endif
@@ -113,6 +128,9 @@ struct echo_can_state {
 	int *a_i;
 	/* ... */
 	short *a_s;
+	/* Backups */
+	int *b_i;
+	int *c_i;
 	/* Reference samples of far-end receive signal */
 	echo_can_cb_s y_s;
 	/* Reference samples of near-end signal */
@@ -140,8 +158,12 @@ struct echo_can_state {
 	int avg_Lu_i_toolow; 
 	int avg_Lu_i_ok;
 #endif 
-	short lastsig[256];
-	int lastpos;
+	short lastsig;
+	int lastcount;
+	int backup;
+#ifdef DC_NORMALIZE
+	int dc_estimate;
+#endif
 
 };
 
@@ -150,6 +172,11 @@ static void echo_can_init(void)
 	printk("Zaptel Echo Canceller: MG2%s\n", ZAPTEL_ECHO_AGGRESSIVE);
 }
 
+static void echo_can_identify(char *buf, size_t len)
+{
+	strncpy(buf, "MG2", len);
+}
+
 static void echo_can_shutdown(void)
 {
 }
@@ -202,6 +229,12 @@ static inline void init_cc(struct echo_can_state *ec, int N, int maxy, int maxu)
 	ec->a_s = ptr;
 	ptr += (sizeof(short) * ec->N_d);
 
+	/* Allocate backup memory */
+	ec->b_i = ptr;
+	ptr += (sizeof(int) * ec->N_d);
+	ec->c_i = ptr;
+	ptr += (sizeof(int) * ec->N_d);
+
 	/* Reset Y circular buffer (short version) */
 	init_cb_s(&ec->y_s, maxy, ptr);
 	ptr += (sizeof(short) * (maxy) * 2);
@@ -247,9 +280,20 @@ static inline void init_cc(struct echo_can_state *ec, int N, int maxy, int maxu)
 
 static inline void echo_can_free(struct echo_can_state *ec)
 {
+#if defined(DC_NORMALIZE) && defined(MEC2_DCBIAS_MESSAGE)
+	printk("EC: DC bias calculated: %d V\n", ec->dc_estimate >> 15);
+#endif
 	FREE(ec);
 }
 
+#ifdef DC_NORMALIZE
+short inline dc_removal(int *dc_estimate, short samp)
+{
+	*dc_estimate += ((((int)samp << 15) - *dc_estimate) >> 9);
+	return samp - (*dc_estimate >> 15);
+}
+#endif
+
 static inline short echo_can_update(struct echo_can_state *ec, short iref, short isig) 
 {
 
@@ -264,6 +308,10 @@ static inline short echo_can_update(struct echo_can_state *ec, short iref, short
 	int Py_i;
 	/* ... */
 	int two_beta_i;
+
+#ifdef DC_NORMALIZE
+	isig = dc_removal(&ec->dc_estimate, isig);
+#endif
 	
 	/* flow A on pg. 428 */
 	/* eq. (16): high-pass filter the input to generate the next value;
@@ -293,37 +341,34 @@ static inline short echo_can_update(struct echo_can_state *ec, short iref, short
   			ec->N_d);
 	rs >>= 15;
 
-	ec->lastsig[ec->lastpos++] = isig;
-	if (ec->lastpos >= 256)
-		ec->lastpos = 0;
-
-	for (k=0; k < 256; k++) {
-		if (isig != ec->lastsig[k])
-			break;
+	if (ec->lastsig == isig) {
+		ec->lastcount++;
+	} else {
+		ec->lastcount = 0;
+		ec->lastsig = isig;
 	}
 
 	if (isig == 0) {
 		u = 0;
-	} else if (k == 256) {
+	} else if (ec->lastcount > 255) {
+		/* We have seen the same input-signal more than 255 times,
+		 * we should pass it through uncancelled, as we are likely on hold */
 		u = isig;
 	} else {
 		if (rs < -32768) {
 			rs = -32768;
 			ec->HCNTR_d = DEFAULT_HANGT;
-			memset(ec->a_i, 0, sizeof(int) * ec->N_d);
-			memset(ec->a_s, 0, sizeof(short) * ec->N_d);
+			RESTORE_COEFFS;
 		} else if (rs > 32767) {
 			rs = 32767;
 			ec->HCNTR_d = DEFAULT_HANGT;
-			memset(ec->a_i, 0, sizeof(int) * ec->N_d);
-			memset(ec->a_s, 0, sizeof(short) * ec->N_d);
+			RESTORE_COEFFS;
 		}
 
-		if (ABS(ABS(rs)-ABS(isig)) > 3000)
+		if (ABS(ABS(rs)-ABS(isig)) > MAX_SIGN_ERROR)
 		{
 			rs = 0;
-			memset(ec->a_i, 0, sizeof(int) * ec->N_d);
-			memset(ec->a_s, 0, sizeof(short) * ec->N_d);
+			RESTORE_COEFFS;
 		}
 
 		/* eq. (3): compute the output value (see figure 3) and the error
@@ -339,6 +384,14 @@ static inline short echo_can_update(struct echo_can_state *ec, short iref, short
 	/* Push a copy of the output value sample into its circular buffer */
 	add_cc_s(&ec->u_s, u);
 
+	if (!ec->backup) {
+		/* Backup coefficients periodically */
+		ec->backup = BACKUP;
+		memcpy(ec->c_i,ec->b_i,ec->N_d*sizeof(int));
+		memcpy(ec->b_i,ec->a_i,ec->N_d*sizeof(int));
+	} else
+		ec->backup--;
+
 
 	/* Update the Near-end hybrid signal circular buffers and accumulators */
 	/* ------------------------------------------------------------------- */
@@ -424,6 +477,7 @@ static inline short echo_can_update(struct echo_can_state *ec, short iref, short
 	    && (ec->max_y_tilde > 0))  {
 		/* Then start the Hangover counter */
 		ec->HCNTR_d = DEFAULT_HANGT;
+		RESTORE_COEFFS;
 #ifdef MEC2_STATS_DETAILED
 		printk(KERN_INFO "Reset near end speech timer with: s_tilde_i %d, stmnt %d, max_y_tilde %d\n", ec->s_tilde_i, (ec->s_tilde_i >> (DEFAULT_ALPHA_ST_I - 1)), ec->max_y_tilde);
 #endif
@@ -446,6 +500,13 @@ static inline short echo_can_update(struct echo_can_state *ec, short iref, short
 		!(ec->i_d % DEFAULT_M)) {		/* we only update on every DEFAULM_M'th sample from the stream */
   			if (ec->Lu_i > MIN_UPDATE_THRESH_I) {	/* there is sufficient energy above the noise floor to contain meaningful data */
   							/* so loop over all the filter coefficients */
+#ifdef USED_COEFFS
+				int max_coeffs[USED_COEFFS];
+				int *pos;
+
+				if (ec->N_d > USED_COEFFS)
+					memset(max_coeffs, 0, USED_COEFFS*sizeof(int));
+#endif
 #ifdef MEC2_STATS_DETAILED
 				printk( KERN_INFO "updating coefficients with: ec->Lu_i %9d\n", ec->Lu_i);
 #endif
@@ -462,7 +523,31 @@ static inline short echo_can_update(struct echo_can_state *ec, short iref, short
 					/* eq. (7): update the coefficient */
 					ec->a_i[k] += grad2 / two_beta_i;
 					ec->a_s[k] = ec->a_i[k] >> 16;
+
+#ifdef USED_COEFFS
+					if (ec->N_d > USED_COEFFS) {
+						if (abs(ec->a_i[k]) > max_coeffs[USED_COEFFS-1]) {
+							/* More or less insertion-sort... */
+							pos = max_coeffs;
+							while (*pos > abs(ec->a_i[k]))
+								pos++;
+
+							if (*pos > max_coeffs[USED_COEFFS-1])
+								memmove(pos+1, pos, (USED_COEFFS-(pos-max_coeffs)-1)*sizeof(int));
+
+							*pos = abs(ec->a_i[k]);
+						}
+					}
+#endif
 				}
+
+#ifdef USED_COEFFS
+				/* Filter out irrelevant coefficients */
+				if (ec->N_d > USED_COEFFS)
+					for (k=0; k < ec->N_d; k++)
+						if (abs(ec->a_i[k]) < max_coeffs[USED_COEFFS-1])
+							ec->a_i[k] = ec->a_s[k] = 0;
+#endif
 		 	 } else { 
 #ifdef MEC2_STATS_DETAILED
 				printk( KERN_INFO "insufficient signal to update coefficients ec->Lu_i %5d < %5d\n", ec->Lu_i, MIN_UPDATE_THRESH_I);
@@ -582,6 +667,8 @@ static inline struct echo_can_state *echo_can_create(int len, int adaption_mode)
 									4 + 						/* align */
 									sizeof(int) * len +			/* a_i */
 									sizeof(short) * len + 		/* a_s */
+									sizeof(int) * len +		/* b_i */
+									sizeof(int) * len +		/* c_i */
 									2 * sizeof(short) * (maxy) +	/* y_s */
 									2 * sizeof(short) * (1 << DEFAULT_ALPHA_ST_I) + /* s_s */
 									2 * sizeof(short) * (maxu) +		/* u_s */
@@ -591,11 +678,13 @@ static inline struct echo_can_state *echo_can_create(int len, int adaption_mode)
 									4 + 						/* align */
 									sizeof(int) * len +			/* a_i */
 									sizeof(short) * len + 		/* a_s */
+									sizeof(int) * len +		/* b_i */
+									sizeof(int) * len +		/* c_i */
 									2 * sizeof(short) * (maxy) +	/* y_s */
 									2 * sizeof(short) * (1 << DEFAULT_ALPHA_ST_I) + /* s_s */
 									2 * sizeof(short) * (maxu) +		/* u_s */
 									2 * sizeof(short) * len);			/* y_tilde_s */
-	  init_cc(ec, len, maxy, maxu);
+		init_cc(ec, len, maxy, maxu);
 	}
 	return ec;
 }
@@ -607,14 +696,20 @@ static inline int echo_can_traintap(struct echo_can_state *ec, int pos, short va
 	 */
 	ec->HCNTR_d = ec->N_d << 1;
 
-	if (pos >= ec->N_d)
+	if (pos >= ec->N_d) {
+		memcpy(ec->b_i,ec->a_i,ec->N_d*sizeof(int));
+		memcpy(ec->c_i,ec->a_i,ec->N_d*sizeof(int));
 		return 1;
+	}
 
 	ec->a_i[pos] = val << 17;
 	ec->a_s[pos] = val << 1;
 
-	if (++pos >= ec->N_d)
+	if (++pos >= ec->N_d) {
+		memcpy(ec->b_i,ec->a_i,ec->N_d*sizeof(int));
+		memcpy(ec->c_i,ec->a_i,ec->N_d*sizeof(int));
 		return 1;
+	}
 
 	return 0;
 }
