@@ -24,7 +24,7 @@ You should have received a copy of the GNU General Public License
 along with the OCT6100 GPL API; if not, write to the Free Software 
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 
-$Octasic_Release: OCT612xAPI-01.00-PR43 $
+$Octasic_Release: OCT612xAPI-01.00-PR47 $
 
 $Octasic_Revision: 22 $
 
@@ -2730,7 +2730,7 @@ UINT32 OctApiLlm2ListRemoveItem(void * l,UINT32 list_handle,UINT32 item_key, PUI
 |	f_ulSize			UINT32			The number of bytes to copy.
 |
 \*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*/
-#if !SKIP_* OctApiLlmMemCpy
+#if !SKIP_OctApiLlmMemCpy
 VOID * OctApiLlmMemCpy( VOID *f_pvDestination, const VOID * f_pvSource, UINT32 f_ulSize )
 {
     CHAR * pbyDst;
