@@ -24,9 +24,9 @@ You should have received a copy of the GNU General Public License
 along with the OCT6100 GPL API; if not, write to the Free Software 
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 
-$Octasic_Release: OCT612xAPI-01.00-PR43 $
+$Octasic_Release: OCT612xAPI-01.00-PR47 $
 
-$Octasic_Revision: 111 $
+$Octasic_Revision: 112 $
 
 \*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*/
 
@@ -1385,7 +1385,21 @@ UINT32 Oct6100ApiInterpretTlvEntry(
 		}
 
 		break;
+	case cOCT6100_TLV_TYPE_RIN_ANR_VALUE:
 		
+		ulResult = Oct6100ApiTlvCheckLengthField( f_ulTlvFieldLength, 
+												  cOCT6100_TLV_MIN_LENGTH_RIN_ANR_VALUE_RW, 
+												  cOCT6100_TLV_MAX_LENGTH_RIN_ANR_VALUE_RW );
+		if ( ulResult == cOCT6100_ERR_OK )
+		{
+			ulResult = Oct6100ApiTlvReadBitOffsetStruct( f_pApiInstance,
+														 f_ulTlvValueAddress, 
+														 &f_pApiInstance->pSharedInfo->MemoryMap.RinAnrValOfst );
+
+			f_pApiInstance->pSharedInfo->ImageInfo.fRoutNoiseReductionLevel = TRUE;
+		}
+
+		break;
 	case cOCT6100_TLV_TYPE_RIN_MUTE_BOFF:
 
 		ulResult = Oct6100ApiTlvCheckLengthField( f_ulTlvFieldLength, 
