@@ -24,7 +24,7 @@ You should have received a copy of the GNU General Public License
 along with the OCT6100 GPL API; if not, write to the Free Software 
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 
-$Octasic_Release: OCT612xAPI-01.00-PR43 $
+$Octasic_Release: OCT612xAPI-01.00-PR47 $
 
 $Octasic_Revision: 20 $
 
@@ -410,13 +410,13 @@ VOID Oct6100ApiCreateFeatureMask(
 				IN		UINT32						f_ulFieldBitOffset,
 				OUT		PUINT32						f_pulFieldMask );
 
-PUINT8 Oct6100ApiStrStr( 
-				IN		PUINT8						f_pszSource, 
-				IN		PUINT8						f_pszString, 
-				IN		PUINT8						f_pszLastCharPtr );
+unsigned char const *Oct6100ApiStrStr( 
+				IN		unsigned char const				*f_pszSource, 
+				IN		unsigned char const				*f_pszString, 
+				IN		unsigned char const				*f_pszLastCharPtr );
 
 UINT32 Oct6100ApiStrLen( 
-				IN		PUINT8						f_pszString );
+				IN		unsigned char const				*f_pszString );
 
 UINT32 Oct6100ApiAsciiToHex( 
 				IN		UINT8						f_chCharacter, 
