@@ -105,6 +105,11 @@ static void echo_can_init(void)
 	printk("Zaptel Echo Canceller: STEVE%s\n", ZAPTEL_ECHO_AGGRESSIVE);
 }
 
+static void echo_can_identify(char *buf, size_t len)
+{
+	strncpy(buf, "STEVE", len);
+}
+
 static void echo_can_shutdown(void)
 {
 }
