@@ -33,10 +33,8 @@
 #include <asm/io.h>
 #ifdef STANDALONE_ZAPATA
 #include "zaptel.h"
-#include "torisa.h"
 #else
-#include <linux/zaptel.h>
-#include <linux/torisa.h>
+#include <zaptel/zaptel.h>
 #endif
 #ifdef LINUX26
 #include <linux/moduleparam.h>
