@@ -35,7 +35,7 @@
 #ifdef STANDALONE_ZAPATA
 #include "zaptel.h"
 #else
-#include <linux/zaptel.h>
+#include <zaptel/zaptel.h>
 #endif
 #ifdef LINUX26
 #include <linux/moduleparam.h>
