@@ -204,7 +204,7 @@ static struct fxo_mode {
 #ifdef STANDALONE_ZAPATA
 #include "zaptel.h"
 #else
-#include <linux/zaptel.h>
+#include <zaptel/zaptel.h>
 #endif
 
 #ifdef LINUX26
@@ -378,6 +378,11 @@ static int _opermode = 0;
 static char *opermode = "FCC";
 static int fxshonormode = 0;
 static int alawoverride = 0;
+static int fastpickup = 0;
+static int fxotxgain = 0;
+static int fxorxgain = 0;
+static int fxstxgain = 0;
+static int fxsrxgain = 0;
 
 static int wctdm_init_proslic(struct wctdm *wc, int card, int fast , int manual, int sane);
 
@@ -1470,7 +1475,11 @@ static int wctdm_init_voicedaa(struct wctdm *wc, int card, int fast, int manual,
 	wctdm_setreg(wc, card, 30, reg30);
 
 	/* Misc. DAA parameters */
-	reg31 = 0xa3;
+	if (fastpickup)
+		reg31 = 0xb3;
+	else
+		reg31 = 0xa3;
+
 	reg31 |= (fxo_modes[_opermode].ohs2 << 3);
 	wctdm_setreg(wc, card, 31, reg31);
 
@@ -1483,6 +1492,9 @@ static int wctdm_init_voicedaa(struct wctdm *wc, int card, int fast, int manual,
 	/* Enable ISO-Cap */
 	wctdm_setreg(wc, card, 6, 0x00);
 
+	if (fastpickup)
+		wctdm_setreg(wc, card, 17, wctdm_getreg(wc, card, 17) | 0x20);
+
 	/* Wait 1000ms for ISO-cap to come up */
 	newjiffies = jiffies;
 	newjiffies += 2 * HZ;
@@ -1499,12 +1511,45 @@ static int wctdm_init_voicedaa(struct wctdm *wc, int card, int fast, int manual,
 		       (wctdm_getreg(wc, card, 13) >> 2) & 0xf);
 	/* Enable on-hook line monitor */
 	wctdm_setreg(wc, card, 5, 0x08);
+	
+	/* Take values for fxotxgain and fxorxgain and apply them to module */
+	if (fxotxgain) {
+		if (fxotxgain >=  -150 && fxotxgain < 0) {
+			wctdm_setreg(wc, card, 38, 16 + (fxotxgain/-10));
+			if(fxotxgain % 10) {
+				wctdm_setreg(wc, card, 40, 16 + (-fxotxgain%10));
+			}
+		}
+		else if (fxotxgain <= 120 && fxotxgain > 0) {
+			wctdm_setreg(wc, card, 38, fxotxgain/10);
+			if(fxotxgain%10) {
+				wctdm_setreg(wc, card, 40, (fxotxgain%10));
+			}
+		}
+	}
+	if (fxorxgain) {
+		if (fxorxgain >=  -150 && fxorxgain < 0) {
+			wctdm_setreg(wc, card, 39, 16 + (fxorxgain/-10));
+			if(fxotxgain%10) {
+				wctdm_setreg(wc, card, 41, 16 + (-fxorxgain%10));
+			}
+		}
+		else if (fxorxgain <= 120 && fxorxgain > 0) {
+			wctdm_setreg(wc, card, 39, fxorxgain/10);
+			if(fxorxgain % 10) {
+				wctdm_setreg(wc, card, 41, (fxorxgain%10));
+			}
+		}
+	}
 
 	/* NZ -- crank the tx gain up by 7 dB */
 	if (!strcmp(fxo_modes[_opermode].name, "NEWZEALAND")) {
 		printk("Adjusting gain\n");
 		wctdm_setreg(wc, card, 38, 0x7);
 	}
+	
+	if(debug)
+		printk("DEBUG fxotxgain:%i.%i fxorxgain:%i.%i\n", (wctdm_getreg(wc, card, 38)/16)?-(wctdm_getreg(wc, card, 38) - 16) : wctdm_getreg(wc, card, 38), (wctdm_getreg(wc, card, 40)/16)? -(wctdm_getreg(wc, card, 40) - 16):wctdm_getreg(wc, card, 40), (wctdm_getreg(wc, card, 39)/16)? -(wctdm_getreg(wc, card, 39) - 16) : wctdm_getreg(wc, card, 39),(wctdm_getreg(wc, card, 41)/16)?-(wctdm_getreg(wc, card, 41) - 16):wctdm_getreg(wc, card, 41));
 
 	return 0;
 		
@@ -1514,7 +1559,7 @@ static int wctdm_init_proslic(struct wctdm *wc, int card, int fast, int manual,
 {
 
 	unsigned short tmp[5];
-	unsigned char r19;
+	unsigned char r19,r9;
 	int x;
 	int fxsmode=0;
 
@@ -1709,6 +1754,38 @@ static int wctdm_init_proslic(struct wctdm *wc, int card, int fast, int manual,
 			printk("Reducing ring power on slot %d (50V peak)\n", card + 1);
 		}
 	}
+
+	if(fxstxgain || fxsrxgain) {
+		r9 = wctdm_getreg(wc, card, 9);
+		switch (fxstxgain) {
+		
+			case 35:
+				r9+=8;
+				break;
+			case -35:
+				r9+=4;
+				break;
+			case 0: 
+				break;
+		}
+	
+		switch (fxsrxgain) {
+			
+			case 35:
+				r9+=2;
+				break;
+			case -35:
+				r9+=1;
+				break;
+			case 0:
+				break;
+		}
+		wctdm_setreg(wc,card,9,r9);
+	}
+
+	if(debug)
+			printk("DEBUG: fxstxgain:%s fxsrxgain:%s\n",((wctdm_getreg(wc, card, 9)/8) == 1)?"3.5":(((wctdm_getreg(wc,card,9)/4) == 1)?"-3.5":"0.0"),((wctdm_getreg(wc, card, 9)/2) == 1)?"3.5":((wctdm_getreg(wc,card,9)%2)?"-3.5":"0.0"));
+
 	wctdm_setreg(wc, card, 64, 0x01);
 	return 0;
 }
@@ -2407,6 +2484,11 @@ module_param(fxshonormode, int, 0600);
 module_param(battdebounce, int, 0600);
 module_param(battthresh, int, 0600);
 module_param(alawoverride, int, 0600);
+module_param(fastpickup, int, 0600);
+module_param(fxotxgain, int, 0600);
+module_param(fxorxgain, int, 0600);
+module_param(fxstxgain, int, 0600);
+module_param(fxsrxgain, int, 0600);
 #else
 MODULE_PARM(debug, "i");
 MODULE_PARM(loopcurrent, "i");
@@ -2422,6 +2504,11 @@ MODULE_PARM(fxshonormode, "i");
 MODULE_PARM(battdebounce, "i");
 MODULE_PARM(battthresh, "i");
 MODULE_PARM(alawoverride, "i");
+MODULE_PARM(fastpickup, "i");
+MODULE_PARM(fxotxgain, "i");
+MODULE_PARM(fxorxgain, "i");
+MODULE_PARM(fxstxgain, "i");
+MODULE_PARM(fxsrxgain, "i");
 #endif
 MODULE_DESCRIPTION("Wildcard TDM400P Zaptel Driver");
 MODULE_AUTHOR("Mark Spencer <markster@digium.com>");
