@@ -45,10 +45,9 @@
 #endif
 
 #ifdef STANDALONE_ZAPATA
-
 #include "zaptel.h"
 #else
-#include <linux/zaptel.h>
+#include <zaptel/zaptel.h>
 #endif /* STANDALONE_ZAPATA */
 
 #include "wcusb.h"
