@@ -23,19 +23,16 @@
  */
 
 #include "xpd.h"
-#include "slic.h"
 
 enum fxo_opcodes {
 	XPROTO_NAME(FXO, SIG_CHANGED)		= 0x06,
 /**/
 	XPROTO_NAME(FXO, DAA_WRITE)		= 0x0F,	/* Write to DAA */
-	XPROTO_NAME(FXO, CHAN_ENABLE)		= 0x0F,	/* Write to DAA */
+	XPROTO_NAME(FXO, XPD_STATE)		= 0x0F,	/* Write to DAA */
 	XPROTO_NAME(FXO, CHAN_CID)		= 0x0F,	/* Write to DAA */
 	XPROTO_NAME(FXO, RING)			= 0x0F,	/* Write to DAA */
 	XPROTO_NAME(FXO, LED)			= 0x0F,	/* Write to DAA */
 	XPROTO_NAME(FXO, RELAY_OUT)		= 0x0F,	/* Write to DAA */
-	XPROTO_NAME(FXO, DAA_INIT)		= 0x0F,	/* Write to DAA */
-	XPROTO_NAME(FXO, DAA_QUERY)		= 0x0F,	/* Write to DAA */
 /**/
 	XPROTO_NAME(FXO, DAA_REPLY)		= 0x10,
 };
@@ -47,11 +44,7 @@ DEF_RPACKET_DATA(FXO, SIG_CHANGED,
 	xpp_line_t	sig_toggles;	/* channels: lsb=1, msb=8 */
 	);
 DEF_RPACKET_DATA(FXO, DAA_REPLY,	/* Get status of a single DAA (for debugging) */
-	xpp_line_t	lines;
-	slic_reply_t	info;
-	);
-DEF_RPACKET_DATA(FXO, DAA_WRITE,
-	slic_cmd_t	slic_cmd;
+	reg_cmd_t	regcmd;
 	);
 
 #define	DAA_VBAT_REGISTER	29
