@@ -46,8 +46,8 @@
 
 #ifdef __KERNEL__
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,14)
-/* pretend every 2.6.9 is later RHEL */
-#if LINUX_VERSION_CODE != KERNEL_VERSION(2,6,9) 
+/* also added in RHEL kernels with the OpenInfiniband backport: */
+#if LINUX_VERSION_CODE != KERNEL_VERSION(2,6,9) || !defined(DEFINE_SPINLOCK)
 typedef	unsigned gfp_t;		/* Added in 2.6.14 */
 #endif
 #endif
@@ -60,12 +60,12 @@ typedef	unsigned gfp_t;		/* Added in 2.6.14 */
 #define	DEF_PARM_BOOL(name,init,perm,desc)	\
 	int name = init;	\
 	module_param(name, bool, perm);		\
-	MODULE_PARM_DESC(name, desc)
+	MODULE_PARM_DESC(name, desc " [default " #init "]")
 
 #define	DEF_PARM(type,name,init,perm,desc)	\
 	type name = init;	\
 	module_param(name, type, perm);		\
-	MODULE_PARM_DESC(name, desc)
+	MODULE_PARM_DESC(name, desc " [default " #init "]")
 
 #if	LINUX_VERSION_CODE	< KERNEL_VERSION(2,6,10)
 /*
@@ -208,8 +208,22 @@ struct xbus {
 	atomic_t		packet_counter;		/* Allocated packets */
 	wait_queue_head_t	packet_cache_empty;
 
+	/* PCM metrics */
+	struct timeval		last_tx_sync;
+	struct timeval		last_rx_sync;
+	unsigned long		max_tx_sync;
+	unsigned long		min_tx_sync;
+	unsigned long		max_rx_sync;
+	unsigned long		min_rx_sync;
+
 	struct xbus_poller	*poller;
 
+	/*
+	 * Sync adjustment
+	 */
+	int			sync_adjustment;
+	long			pll_updated_at;
+
 	struct	rw_semaphore	in_use;
 	int			num_xpds;
 	void			*priv;			/* Pointer to transport level data structures */
@@ -324,6 +338,14 @@ struct xpd {
 };
 
 #define	for_each_line(xpd,i)	for((i) = 0; (i) < (xpd)->channels; (i)++)
+#define	IS_BRI(xpd)		((xpd)->type == XPD_TYPE_BRI_NT || (xpd)->type == XPD_TYPE_BRI_TE)
+#define	TICK_TOLERANCE		500 /* usec */
+
+#ifdef	DEBUG_SYNC_PARPORT
+void xbus_flip_bit(xbus_t *xbus, unsigned int bitnum0, unsigned int bitnum1);
+#else
+#define	xbus_flip_bit(xbus, bitnum0, bitnum1)
+#endif
 
 #endif
 
