@@ -54,7 +54,8 @@ typedef	byte	xpd_type_t;
 typedef struct xpd_addr {
 	byte	unit:UNIT_BITS;
 	byte	subunit:SUBUNIT_BITS;
-	byte	reserved:(8-UNIT_BITS-SUBUNIT_BITS);
+	byte	reserved:1;
+	byte	sync_master:1;
 } PACKED xpd_addr_t;
 
 bool valid_xpd_addr(const xpd_addr_t *addr);
