@@ -6,7 +6,7 @@
  * written by Jim Dixon <jim@lambdatel.com>.
  *
  * Copyright (C) 2001 Jim Dixon / Zapata Telephony.
- * Copyright (C) 2001, Linux Support Services, Inc.
+ * Copyright (C) 2001 - 2006 Digium, Inc.
  *
  * All rights reserved.
  *
@@ -28,15 +28,14 @@
 #ifndef _LINUX_ZAPTEL_H
 #define _LINUX_ZAPTEL_H
 
-#include <linux/types.h>
-
 #ifdef __KERNEL__
 #include "zconfig.h"
 #include <linux/version.h>
-
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,18)
 #include <linux/config.h>
 #endif
+#include <linux/fs.h>
+#include <linux/ioctl.h>
 
 #ifdef CONFIG_ZAPATA_NET	
 #include <linux/hdlc.h>
@@ -48,8 +47,6 @@
 #include <linux/interrupt.h>
 #endif
 
-#include <linux/fs.h>
-
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
 #define LINUX26
 #endif
@@ -72,6 +69,7 @@
 
 #include "ecdis.h"
 #include "fasthdlc.h"
+
 #endif /* __KERNEL__ */
 
 #ifdef CONFIG_DEVFS_FS
@@ -82,7 +80,6 @@
 //#warning "Zaptel doesn't support DEVFS in post 2.4 kernels.  Disabling DEVFS in zaptel"
 #endif
 #endif /* CONFIG_DEVFS_FS */
-#include <linux/ioctl.h>
 
 #ifndef ELAST
 #define ELAST 500
@@ -132,6 +129,7 @@
 #define ZT_SIG_DACS	(__ZT_SIG_DACS | ZT_SIG_CLEAR)	/* Cross connect */
 #define ZT_SIG_EM_E1	 (1 << 17)			/* E1 E&M Variation */
 #define ZT_SIG_DACS_RBS	 ((1 << 18) | __ZT_SIG_DACS)	/* Cross connect w/ RBS */
+#define ZT_SIG_HARDHDLC	((1 << 19) | ZT_SIG_CLEAR)
 
 /* tone flag values */
 #define	ZT_REVERSE_RXTONE 1  /* reverse polarity rx tone logic */
@@ -255,6 +253,7 @@ int	deflaw;		/* Default law (ZT_LAW_DEFAULT, ZT_LAW_MULAW, or ZT_LAW_ALAW */
 int	master;		/* Master channel if sigtype is ZT_SLAVE */
 int	idlebits;	/* Idle bits (if this is a CAS channel) or
 			   channel to monitor (if this is DACS channel) */
+char netdev_name[16]; /*name for the hdlc network device*/
 } ZT_CHANCONFIG;
 
 typedef struct zt_sfconfig
@@ -321,6 +320,11 @@ int	op;
 void	*data;
 } ZT_INDIRECT_DATA;	
 
+struct zt_versioninfo {
+	char version[80];
+	char echo_canceller[80];
+};
+
 
 /* ioctl definitions */
 #define ZT_CODE	'J'
@@ -613,6 +617,18 @@ void	*data;
 #define ZT_INDIRECT _IOWR (ZT_CODE, 56, struct zt_indirect_data)
 
 
+/*
+ * Get the version of Zaptel that is running, and a description
+ * of the compiled-in echo canceller (if any)
+ */
+#define ZT_GETVERSION _IOR(ZT_CODE, 57, struct zt_versioninfo)
+
+/*
+ * Put the channel in loopback mode (receive from the channel is
+ * transmitted back on the interface)
+ */
+#define ZT_LOOPBACK _IOW(ZT_CODE, 58, int)
+
 /*
  *  60-80 are reserved for private drivers
  *  80-85 are reserved for dynamic span stuff
@@ -640,8 +656,8 @@ void	*data;
 #define	ZT_SETPOLARITY		_IOW (ZT_CODE, 92, int)
 
 /*
-* Transcoder operations
-*/
+ * Transcoder operations
+ */
 #define ZT_TRANSCODE_OP		_IOWR(ZT_CODE, 93, int)
 
 /* 
@@ -668,6 +684,32 @@ void	*data;
 #define ZT_TONE_STUTTER		10
 #define ZT_TONE_MAX		16
 
+#define ZT_TONE_DTMF_BASE	64
+
+/*
+ * These must be in the same order as the dtmf_tones array in tones.h 
+ */
+enum {
+	ZT_TONE_DTMF_0 = ZT_TONE_DTMF_BASE,
+	ZT_TONE_DTMF_1,
+	ZT_TONE_DTMF_2,
+	ZT_TONE_DTMF_3,
+	ZT_TONE_DTMF_4,
+	ZT_TONE_DTMF_5,
+	ZT_TONE_DTMF_6,
+	ZT_TONE_DTMF_7,
+	ZT_TONE_DTMF_8,
+	ZT_TONE_DTMF_9,
+	ZT_TONE_DTMF_s,
+	ZT_TONE_DTMF_p,
+	ZT_TONE_DTMF_A,
+	ZT_TONE_DTMF_B,
+	ZT_TONE_DTMF_C,
+	ZT_TONE_DTMF_D
+};
+
+#define ZT_TONE_DTMF_MAX ZT_TONE_DTMF_D
+
 #define ZT_MAX_CADENCE		16
 
 #define ZT_TONEDETECT_ON	(1 << 0)		/* Detect tones */
@@ -725,7 +767,7 @@ typedef struct zt_transcode_header {
 } ZT_TRANSCODE_HEADER;
 
 struct zt_ring_cadence {
-	int ringcadence [ZT_MAX_CADENCE];
+	int ringcadence[ZT_MAX_CADENCE];
 };
 
 struct zt_tone_def_header {
@@ -771,6 +813,9 @@ struct zt_tone_def {		/* Structure for zone programming */
 
 };
 
+#ifdef __KERNEL__
+#endif /* KERNEL */
+
 /* Define the maximum block size */
 #define	ZT_MAX_BLOCKSIZE	8192
 
@@ -865,11 +910,17 @@ struct zt_tone_def {		/* Structure for zone programming */
 /* Timer event -- ping ready */
 #define ZT_EVENT_TIMER_PING		16
 
+/* Polarity reversal event */
+#define ZT_EVENT_POLARITY  17
+
 /* Ring Begin event */
 #define ZT_EVENT_RINGBEGIN  18
 
-/* Polarity reversal event */
-#define ZT_EVENT_POLARITY  17
+/* Echo can disabled event */
+#define ZT_EVENT_EC_DISABLED 19
+
+/* Channel was disconnected. Hint user to close channel */
+#define ZT_EVENT_REMOVED   20
 
 #define ZT_EVENT_PULSEDIGIT (1 << 16)	/* This is OR'd with the digit received */
 #define ZT_EVENT_DTMFDOWN  (1 << 17)	/* Ditto for DTMF key down event */
@@ -947,7 +998,13 @@ struct zt_tone_def {		/* Structure for zone programming */
 #define	ZT_DEFAULT_PULSEAFTERTIME 750	/* 750ms between dial pulse digits */
 
 #define	ZT_MINPULSETIME (15 * 8)	/* 15 ms minimum */
+
+#ifdef SHORT_FLASH_TIME
+#define	ZT_MAXPULSETIME (80 * 8)	/* we need 80 ms, not 200ms, as we have a short flash */
+#else
 #define	ZT_MAXPULSETIME (200 * 8)	/* 200 ms maximum */
+#endif
+
 #define	ZT_PULSETIMEOUT ((ZT_MAXPULSETIME / 8) + 50)
 
 #define ZT_RINGTRAILER (50 * 8)	/* Don't consider a ring "over" until it's been gone at least this
@@ -973,6 +1030,8 @@ struct zt_tone_def {		/* Structure for zone programming */
 #endif
 
 #ifdef __KERNEL__
+
+#include <linux/types.h>
 #include <linux/poll.h>
 
 #define	ZT_MAX_EVENTSIZE	64	/* 64 events max in buffer */
@@ -999,7 +1058,7 @@ struct zt_hdlc {
 #endif
 	struct zt_chan *chan;
 };
-#endif /* CONFIG_ZAPATA_NET */
+#endif
 
 /* Echo cancellation */
 struct echo_can_state;
@@ -1007,6 +1066,7 @@ struct echo_can_state;
 /* echo can API consists of these functions */
 void echo_can_init(void);
 void echo_chan_shutdown(void);
+void echo_can_identify(char *buf, size_t len);
 struct echo_can_state *echo_can_create(int len, int adaption_mode);
 void echo_can_free(struct echo_can_state *ec);
 short echo_can_update(struct echo_can_state *ec, short iref, short isig);
@@ -1073,7 +1133,7 @@ struct zt_chan {
 	u_char *readchunk;						/* Actual place to read from */
 	u_char sreadchunk[ZT_MAX_CHUNKSIZE];	/* Preallocated static area */
 	short *readchunkpreec;
-	
+
 	/* Pointer to tx and rx gain tables */
 	u_char *rxgain;
 	u_char *txgain;
@@ -1283,6 +1343,8 @@ typedef enum {
 #define ZT_FLAG_PPP			(1 << 14)	/* PPP is available */
 #define ZT_FLAG_T1PPP			(1 << 15)
 #define ZT_FLAG_SIGFREEZE		(1 << 16)	/* Freeze signalling */
+#define ZT_FLAG_NOSTDTXRX		(1 << 17)	/* Do NOT do standard transmit and receive on every interrupt */
+#define ZT_FLAG_LOOPED			(1 << 18)	/* Loopback the receive data from the channel to the transmit */
 
 struct zt_span {
 	spinlock_t lock;
@@ -1369,6 +1431,9 @@ struct zt_span {
 	/* Opt: Dacs the contents of chan2 into chan1 if possible */
 	int (*dacs)(struct zt_chan *chan1, struct zt_chan *chan2);
 
+	/* Opt: Used to tell an onboard HDLC controller that there is data ready to transmit */
+	void (*hdlc_hard_xmit)(struct zt_chan *chan);
+
 	/* Used by zaptel only -- no user servicable parts inside */
 	int spanno;			/* Span number for zaptel */
 	int offset;			/* Offset within a given card */
@@ -1441,6 +1506,9 @@ struct zt_dynamic_driver {
 	/* Transmit a given message */
 	int (*transmit)(void *tpipe, unsigned char *msg, int msglen);
 
+	/* Flush any pending messages */
+	int (*flush)(void);
+
 	struct zt_dynamic_driver *next;
 };
 
@@ -1460,10 +1528,25 @@ int zt_receive(struct zt_span *span);
 /* Prepare writechunk buffers on all channels for this span */
 int zt_transmit(struct zt_span *span);
 
+/* Abort the buffer currently being receive with event "event" */
+void zt_hdlc_abort(struct zt_chan *ss, int event);
+
+/* Indicate to zaptel that the end of frame was received and rotate buffers */
+void zt_hdlc_finish(struct zt_chan *ss);
+
+/* Put a chunk of data into the current receive buffer */
+void zt_hdlc_putbuf(struct zt_chan *ss, unsigned char *rxb, int bytes);
+
+/* Get a chunk of data from the current transmit buffer.  Returns -1 if no data
+ * is left to send, 0 if there is data remaining in the current message to be sent
+ * and 1 if the currently transmitted message is now done */
+int zt_hdlc_getbuf(struct zt_chan *ss, unsigned char *bufptr, unsigned int *size);
+
+
 /* Register a span.  Returns 0 on success, -1 on failure.  Pref-master is non-zero if
    we should have preference in being the master device */
 int zt_register(struct zt_span *span, int prefmaster);
- 
+
 /* Allocate / free memory for a transcoder */
 struct zt_transcoder *zt_transcoder_alloc(int numchans);
 void zt_transcoder_free(struct zt_transcoder *ztc);
@@ -1723,4 +1806,25 @@ struct zt_radio_param {
 /*! Maximum audio mask */
 #define ZT_FORMAT_AUDIO_MASK	((1 << 16) - 1)
 
+#define	ZT_RADPAR_DEEMP 18 /* Audio De-empahsis (on or off) */ 
+
+#define	ZT_RADPAR_PREEMP 19 /* Audio Pre-empahsis (on or off) */ 
+
+#define	ZT_RADPAR_RXGAIN 20 /* Audio (In to system) Rx Gain */ 
+
+#define	ZT_RADPAR_TXGAIN 21 /* Audio (Out from system) Tx Gain */ 
+
+struct torisa_debug {
+	unsigned int txerrors;
+	unsigned int irqcount;
+	unsigned int taskletsched;
+	unsigned int taskletrun;
+	unsigned int taskletexec;
+	int span1flags;
+	int span2flags;
+};
+
+/* Special torisa ioctl */
+#define TORISA_GETDEBUG		_IOW (ZT_CODE, 60, struct torisa_debug)
+
 #endif /* _LINUX_ZAPTEL_H */
