@@ -534,7 +534,65 @@ struct tone_zone builtin_zones[]  =
 		  { ZT_TONE_STUTTER, "450+425" },
 	  },
 	},
-        { 34, "ve", "Venezuela", { 1000, 4000 },
+	{ 34, "ar", "Argentina", { 1000, 4500 },
+	  {
+		{ ZT_TONE_DIALTONE, "425" },
+		{ ZT_TONE_BUSY, "425/300,0/300" },
+		{ ZT_TONE_RINGTONE, "425/1000,0/4500" },
+		{ ZT_TONE_CONGESTION, "425/200,0/300" },
+		{ ZT_TONE_CALLWAIT, "425/200,0/9000" },
+		{ ZT_TONE_DIALRECALL, "!425/100,!0/100,!425/100,!0/100,!425/100,!0/100,425/330,0/330,425/660,0/660" },
+		{ ZT_TONE_RECORDTONE, "1400/500,0/14000" },
+		{ ZT_TONE_INFO, "425/100,0/100" },
+		{ ZT_TONE_STUTTER, "425/450,0/50" },
+	  },
+	},
+	{ 35, "my", "Malaysia", { 400, 200, 400, 2000 },
+  	  {
+        	{ ZT_TONE_DIALTONE, "425" },
+		{ ZT_TONE_BUSY, "425/500,0/500" },
+		{ ZT_TONE_RINGTONE, "425/400,0/200,425/400,0/2000" },
+		{ ZT_TONE_CONGESTION, "425/500,0/500" },
+		{ ZT_TONE_CALLWAIT, "425/100,0/4000" },
+		{ ZT_TONE_DIALRECALL, "350+440" },
+		{ ZT_TONE_RECORDTONE, "1400/500,0/60000" },
+		{ ZT_TONE_INFO, "950/330,0/15,1400/330,0/15,1800/330,0/1000" },
+		{ ZT_TONE_STUTTER, "450+425" },
+	  },
+	},
+	{ 36, "th", "Thailand", { 1000, 4000 },
+          {
+		/* Reference: http://www.itu.int/ITU-T/inr/forms/files/tones-0203.pdf */
+		{ ZT_TONE_DIALTONE,   "400*50" },
+		{ ZT_TONE_BUSY,       "400/500,0/500" },
+		{ ZT_TONE_RINGTONE,   "400/1000,0/4000" },
+		{ ZT_TONE_CONGESTION, "400/300,0/300" },
+		{ ZT_TONE_CALLWAIT,   "1000/400,10000/400,1000/400" },
+		/* DIALRECALL - not specified - use special dial tone instead. */
+		{ ZT_TONE_DIALRECALL, "400*50/400,0/100,400*50/400,0/100" },
+		/* RECORDTONE - not specified */
+		{ ZT_TONE_RECORDTONE, "1400/500,0/15000" },
+		/* INFO - specified as an announcement - use tones instead. */
+		{ ZT_TONE_INFO,       "950/330,1400/330,1800/330" },
+		/* STUTTER - not specified */
+		{ ZT_TONE_STUTTER,    "!400/200,!0/200,!400/600,!0/200,!400/200,!0/200,!400/600,!0/200,!400/200,!0/200,!400/600,!0/200,!400/200,!0/200,!400/600,!0/200,400" },
+	  },
+	},
+	{ 37, "bg", "Bulgaria", { 1000, 4000 },
+          {
+		/* Reference: http://www.itu.int/ITU-T/inr/forms/files/tones-0203.pdf */
+		{ ZT_TONE_DIALTONE,   "425" },
+		{ ZT_TONE_BUSY,       "425/500,0/500" },
+		{ ZT_TONE_RINGTONE,   "425/1000,0/4000" },
+		{ ZT_TONE_CONGESTION, "425/250,0/250" },
+		{ ZT_TONE_CALLWAIT,   "425/150,0/150,425/150,0/4000" },
+		{ ZT_TONE_DIALRECALL, "!425/100,!0/100,!425/100,!0/100,!425/100,!0/100,425" },
+		{ ZT_TONE_RECORDTONE, "1400/425,0/15000" },
+		{ ZT_TONE_INFO,       "950/330,1400/330,1800/330,0/1000" },
+		{ ZT_TONE_STUTTER,    "425/1500,0/100" },
+	  },
+	},
+        { 38, "ve", "Venezuela", { 1000, 4000 },
           {
             /* Reference: http://www.itu.int/ITU-T/inr/forms/files/tones-0203.pdf */
             { ZT_TONE_DIALTONE, "425" },
