@@ -38,9 +38,14 @@
 #include <fcntl.h>
 #include <errno.h>
 #include <math.h>
-#include "ztcfg.h"
-#include "tonezone.h"
+#ifdef STANDALONE_ZAPATA
 #include "zaptel.h"
+#include "tonezone.h"
+#else
+#include <zaptel/zaptel.h>
+#include <zaptel/tonezone.h>
+#endif
+#include "ztcfg.h"
 
 #define NUM_SPANS ZT_MAX_SPANS
 
