@@ -37,9 +37,14 @@
 #include <sys/ioctl.h>
 #include <fcntl.h>
 #include <errno.h>
-#include "ztcfg.h"
-#include "tonezone.h"
+#ifdef STANDALONE_ZAPATA
 #include "zaptel.h"
+#include "tonezone.h"
+#else
+#include <zaptel/zaptel.h>
+#include <zaptel/tonezone.h>
+#endif
+#include "ztcfg.h"
 
 #define NUM_SPANS ZT_MAX_SPANS
 
@@ -56,6 +61,7 @@ static char *filename=CONFIG_FILENAME;
 
 int rxtones[NUM_TONES + 1],rxtags[NUM_TONES + 1],txtones[NUM_TONES + 1];
 int bursttime = 0, debouncetime = 0, invertcor = 0, exttone = 0, corthresh = 0;
+int txgain = 0, rxgain = 0, deemp = 0, preemp = 0;
 
 int corthreshes[] = {3125,6250,9375,12500,15625,18750,21875,25000,0} ;
 
@@ -152,6 +158,8 @@ static const char *sigtype_to_str(const int sig)
 		return "Network HDLC";
 	case ZT_SIG_HDLCFCS:
 		return "HDLC with FCS check";
+	case ZT_SIG_HARDHDLC:
+		return "Hardware assisted D-channel";
 	default:
 		return "Unknown";
 	}
@@ -177,6 +185,10 @@ static void clear_fields()
 	debouncetime = 0;
 	invertcor = 0;
 	exttone = 0;
+	txgain = 0;
+	rxgain = 0;
+	deemp = 0;
+	preemp = 0;
 }
 
 static int error(char *fmt, ...)
@@ -552,11 +564,15 @@ static int chanconfig(char *keyword, char *args)
 				}
 			} else if (!strcasecmp(keyword, "nethdlc")) {
 				sig[x] = sigtype_to_str(ZT_SIG_HDLCNET);
+				memset(cc[x].netdev_name, 0, sizeof(cc[x].netdev_name));
 				if (master) {
 					cc[x].sigtype = ZT_SIG_SLAVE;
 					cc[x].master = master;
 				} else {
 					cc[x].sigtype = ZT_SIG_HDLCNET;
+					if (idle) {
+					    strncpy(cc[x].netdev_name, idle, sizeof(cc[x].netdev_name) - 1);
+					}
 					master = x;
 				}
 			} else if (!strcasecmp(keyword, "fcshdlc")) {
@@ -571,6 +587,9 @@ static int chanconfig(char *keyword, char *args)
 			} else if (!strcasecmp(keyword, "dchan")) {
 				sig[x] = "D-channel";
 				cc[x].sigtype = ZT_SIG_HDLCFCS;
+			} else if (!strcasecmp(keyword, "hardhdlc")) {
+				sig[x] = "Hardware assisted D-channel";
+				cc[x].sigtype = ZT_SIG_HARDHDLC;
 			} else {
 				fprintf(stderr, "Huh? (%s)\n", keyword);
 			}
@@ -776,6 +795,86 @@ int burst_time(char *keyword, char *args)
 	return 0;
 }
 
+int tx_gain(char *keyword, char *args)
+{
+	static char *realargs[10];
+	int argc;
+	int res;
+	int val;
+	argc = res = parseargs(args, realargs, 1, ',');
+	if (res != 1) {
+		error("Incorrect number of arguments to 'txgain' (should be <value>)\n");
+	}
+	res = sscanf(realargs[0], "%d", &val);
+	if (res != 1) {
+		error("Invalid value '%s', should be a number > 0.\n", realargs[0]);
+	}
+
+	txgain = val;
+	return 0;
+}
+
+int rx_gain(char *keyword, char *args)
+{
+	static char *realargs[10];
+	int argc;
+	int res;
+	int val;
+	argc = res = parseargs(args, realargs, 1, ',');
+	if (res != 1) {
+		error("Incorrect number of arguments to 'rxgain' (should be <value>)\n");
+	}
+	res = sscanf(realargs[0], "%d", &val);
+	if (res != 1) {
+		error("Invalid value '%s', should be a number > 0.\n", realargs[0]);
+	}
+
+	rxgain = val;
+	return 0;
+}
+
+int de_emp(char *keyword, char *args)
+{
+	static char *realargs[10];
+	int argc;
+	int res;
+	int val;
+	argc = res = parseargs(args, realargs, 1, ',');
+	if (res != 1) {
+		error("Incorrect number of arguments to 'de-emp' (should be <value>)\n");
+	}
+	res = sscanf(realargs[0], "%d", &val);
+	if ((res == 1) && (val < 1))
+		res = -1;
+	if (res != 1) {
+		error("Invalid value '%s', should be a number > 0.\n", realargs[0]);
+	}
+
+	deemp = val;
+	return 0;
+}
+
+int pre_emp(char *keyword, char *args)
+{
+	static char *realargs[10];
+	int argc;
+	int res;
+	int val;
+	argc = res = parseargs(args, realargs, 1, ',');
+	if (res != 1) {
+		error("Incorrect number of arguments to 'pre_emp' (should be <value>)\n");
+	}
+	res = sscanf(realargs[0], "%d", &val);
+	if ((res == 1) && (val < 1))
+		res = -1;
+	if (res != 1) {
+		error("Invalid value '%s', should be a number > 0.\n", realargs[0]);
+	}
+
+	preemp = val;
+	return 0;
+}
+
 int invert_cor(char *keyword, char *args)
 {
 	static char *realargs[10];
@@ -927,55 +1026,57 @@ static int rad_chanconfig(char *keyword, char *args)
 		if (chans[x]) {
 			p.radpar = ZT_RADPAR_NUMTONES;
 			if (ind_ioctl(x,fd,ZT_RADIO_GETPARAM,&p) == -1)
-				error("Cannot get number of tones for channel %d\n",x);
-			n = p.data;
-			p.radpar = ZT_RADPAR_INITTONE;
-			if (ind_ioctl(x,fd,ZT_RADIO_SETPARAM,&p) == -1)
-				error("Cannot init tones for channel %d\n",x);
-			if (!rxtones[0]) for(i = 1; i <= n; i++)
+				n = 0; else n = p.data;
+			if (n)
 			{
-				if (rxtones[i])
+				p.radpar = ZT_RADPAR_INITTONE;
+				if (ind_ioctl(x,fd,ZT_RADIO_SETPARAM,&p) == -1)
+					error("Cannot init tones for channel %d\n",x);
+				if (!rxtones[0]) for(i = 1; i <= n; i++)
 				{
-					p.radpar = ZT_RADPAR_RXTONE;
-					p.index = i;
-					p.data = rxtones[i];
-					if (ind_ioctl(x,fd,ZT_RADIO_SETPARAM,&p) == -1)
-						error("Cannot set rxtone on channel %d\n",x);
+					if (rxtones[i])
+					{
+						p.radpar = ZT_RADPAR_RXTONE;
+						p.index = i;
+						p.data = rxtones[i];
+						if (ind_ioctl(x,fd,ZT_RADIO_SETPARAM,&p) == -1)
+							error("Cannot set rxtone on channel %d\n",x);
+					}
+					if (rxtags[i])
+					{
+						p.radpar = ZT_RADPAR_RXTONECLASS;
+						p.index = i;
+						p.data = rxtags[i];
+						if (ind_ioctl(x,fd,ZT_RADIO_SETPARAM,&p) == -1)
+							error("Cannot set rxtag on channel %d\n",x);
+					}
+					if (txtones[i])
+					{
+						p.radpar = ZT_RADPAR_TXTONE;
+						p.index = i;
+						p.data = txtones[i];
+						if (ind_ioctl(x,fd,ZT_RADIO_SETPARAM,&p) == -1)
+							error("Cannot set txtone on channel %d\n",x);
+					}
+				} else { /* if we may have DCS receive */
+					if (rxtones[0])
+					{
+						p.radpar = ZT_RADPAR_RXTONE;
+						p.index = 0;
+						p.data = rxtones[0];
+						if (ind_ioctl(x,fd,ZT_RADIO_SETPARAM,&p) == -1)
+							error("Cannot set DCS rxtone on channel %d\n",x);
+					}
 				}
-				if (rxtags[i])
-				{
-					p.radpar = ZT_RADPAR_RXTONECLASS;
-					p.index = i;
-					p.data = rxtags[i];
-					if (ind_ioctl(x,fd,ZT_RADIO_SETPARAM,&p) == -1)
-						error("Cannot set rxtag on channel %d\n",x);
-				}
-				if (txtones[i])
+				if (txtones[0])
 				{
 					p.radpar = ZT_RADPAR_TXTONE;
-					p.index = i;
-					p.data = txtones[i];
-					if (ind_ioctl(x,fd,ZT_RADIO_SETPARAM,&p) == -1)
-						error("Cannot set txtone on channel %d\n",x);
-				}
-			} else { /* if we may have DCS receive */
-				if (rxtones[0])
-				{
-					p.radpar = ZT_RADPAR_RXTONE;
 					p.index = 0;
-					p.data = rxtones[0];
+					p.data = txtones[0];
 					if (ind_ioctl(x,fd,ZT_RADIO_SETPARAM,&p) == -1)
-						error("Cannot set DCS rxtone on channel %d\n",x);
+						error("Cannot set default txtone on channel %d\n",x);
 				}
 			}
-			if (txtones[0])
-			{
-				p.radpar = ZT_RADPAR_TXTONE;
-				p.index = 0;
-				p.data = txtones[0];
-				if (ind_ioctl(x,fd,ZT_RADIO_SETPARAM,&p) == -1)
-					error("Cannot set default txtone on channel %d\n",x);
-			}
 			if (debouncetime)
 			{
 				p.radpar = ZT_RADPAR_DEBOUNCETIME;
@@ -990,20 +1091,24 @@ static int rad_chanconfig(char *keyword, char *args)
 				if (ind_ioctl(x,fd,ZT_RADIO_SETPARAM,&p) == -1)
 					error("Cannot set bursttime on channel %d\n",x);
 			}
-			if (invertcor)
-			{
-				p.radpar = ZT_RADPAR_INVERTCOR;
-				p.data = invertcor;
-				if (ind_ioctl(x,fd,ZT_RADIO_SETPARAM,&p) == -1)
-					error("Cannot set invertcor on channel %d\n",x);
-			}
-			if (exttone)
-			{
-				p.radpar = ZT_RADPAR_EXTRXTONE;
-				p.data = exttone;
-				if (ind_ioctl(x,fd,ZT_RADIO_SETPARAM,&p) == -1)
-					error("Cannot set exttone on channel %d\n",x);
-			}
+			p.radpar = ZT_RADPAR_DEEMP;
+			p.data = deemp;
+			ind_ioctl(x,fd,ZT_RADIO_SETPARAM,&p);
+			p.radpar = ZT_RADPAR_PREEMP;
+			p.data = preemp;
+			ind_ioctl(x,fd,ZT_RADIO_SETPARAM,&p);
+			p.radpar = ZT_RADPAR_TXGAIN;
+			p.data = txgain;
+			ind_ioctl(x,fd,ZT_RADIO_SETPARAM,&p);
+			p.radpar = ZT_RADPAR_RXGAIN;
+			p.data = rxgain;
+			ind_ioctl(x,fd,ZT_RADIO_SETPARAM,&p);
+			p.radpar = ZT_RADPAR_INVERTCOR;
+			p.data = invertcor;
+			ind_ioctl(x,fd,ZT_RADIO_SETPARAM,&p);
+			p.radpar = ZT_RADPAR_EXTRXTONE;
+			p.data = exttone;
+			ind_ioctl(x,fd,ZT_RADIO_SETPARAM,&p);
 			if (corthresh)
 			{
 				p.radpar = ZT_RADPAR_CORTHRESH;
@@ -1019,13 +1124,23 @@ static int rad_chanconfig(char *keyword, char *args)
 
 /* End Radio functions */
 
-static void printconfig()
+static void printconfig(int fd)
 {
 	int x,y;
 	int ps;
 	int configs=0;
-	printf("\nZaptel Configuration\n"
-	       "======================\n\n");
+	struct zt_versioninfo vi;
+
+	strcpy(vi.version, "Unknown");
+	strcpy(vi.echo_canceller, "Unknown");
+
+	if (ioctl(fd, ZT_GETVERSION, &vi))
+		error("Unable to read Zaptel version information.\n");
+
+	printf("\nZaptel Version: %s\n"
+	       "Echo Canceller: %s\n"
+	       "Configuration\n"
+	       "======================\n\n", vi.version, vi.echo_canceller);
 	for (x=0;x<spans;x++) 
 		printf("SPAN %d: %3s/%4s Build-out: %s\n",
 		       x+1, ( lc[x].lineconfig & ZT_CONFIG_D4 ? "D4" :
@@ -1084,6 +1199,7 @@ static struct handler {
 	{ "rawhdlc", chanconfig },
 	{ "nethdlc", chanconfig },
 	{ "fcshdlc", chanconfig },
+	{ "hardhdlc", chanconfig },
 	{ "dchan", chanconfig },
 	{ "bchan", chanconfig },
 	{ "indclear", chanconfig },
@@ -1105,6 +1221,10 @@ static struct handler {
 	{ "exttone", ext_tone },
 	{ "invertcor", invert_cor },
 	{ "corthresh", cor_thresh },
+	{ "rxgain", rx_gain },
+	{ "txgain", tx_gain },
+	{ "deemp", de_emp },
+	{ "preemp", pre_emp },
 	{ "channel", rad_chanconfig },
 	{ "channels", rad_chanconfig },
 };
@@ -1226,144 +1346,139 @@ int main(int argc, char *argv[])
 
 	if (!errcnt) {
 		if (verbose) {
-			printconfig();
+			printconfig(fd);
 		}
 		if (fo_real) {
 			if (debug & DEBUG_APPLY) {
 				printf("About to open Master device\n");
 				fflush(stdout);
 			}
-			if (fd == -1) fd = open(MASTER_DEVICE, O_RDWR);
-			if (fd < 0) 
-				error("Unable to open master device '%s'\n", MASTER_DEVICE);
-			else {
-				for (x=0;x<numdynamic;x++) {
-					/* destroy them all */
-					ioctl(fd, ZT_DYNAMIC_DESTROY, &zds[x]);
+			for (x=0;x<numdynamic;x++) {
+				/* destroy them all */
+				ioctl(fd, ZT_DYNAMIC_DESTROY, &zds[x]);
+			}
+			if (stopmode) {
+				for (x=0;x<spans;x++) {
+					if (ioctl(fd, ZT_SHUTDOWN, &lc[x].span)) {
+						fprintf(stderr, "Zaptel shutdown failed: %s\n", strerror(errno));
+						close(fd);
+						exit(1);
+					}
 				}
-				if (stopmode) {
-					for (x=0;x<spans;x++) {
-						if (ioctl(fd, ZT_SHUTDOWN, &lc[x].span)) {
-							fprintf(stderr, "Zaptel shutdown failed: %s\n", strerror(errno));
-							close(fd);
-							exit(1);
-						}
+			} else {
+				for (x=0;x<spans;x++) {
+					if (ioctl(fd, ZT_SPANCONFIG, lc + x)) {
+						fprintf(stderr, "ZT_SPANCONFIG failed on span %d: %s (%d)\n", lc[x].span, strerror(errno), errno);
+						close(fd);
+						exit(1);
 					}
-				} else {
-					for (x=0;x<spans;x++) {
-						if (ioctl(fd, ZT_SPANCONFIG, lc + x)) {
-							fprintf(stderr, "ZT_SPANCONFIG failed on span %d: %s (%d)\n", lc[x].span, strerror(errno), errno);
-							close(fd);
-							exit(1);
-						}
-					}
-					for (x=0;x<numdynamic;x++) {
-						if (ioctl(fd, ZT_DYNAMIC_CREATE, &zds[x])) {
-							fprintf(stderr, "Zaptel dynamic span creation failed: %s\n", strerror(errno));
-							close(fd);
-							exit(1);
-						}
-					}
-					for (x=1;x<ZT_MAX_CHANNELS;x++) {
-						struct zt_params current_state;
-						int master;
-						int needupdate = force;
-
-						if (debug & DEBUG_APPLY) {
-							printf("Configuring device %d\n", x);
-							fflush(stdout);
-						}
-						if (!cc[x].sigtype)
-							continue;
-
-						if (!needupdate) {
-							memset(&current_state, 0, sizeof(current_state));
-							current_state.channo = cc[x].chan | ZT_GET_PARAMS_RETURN_MASTER;
-							if (ioctl(fd, ZT_GET_PARAMS, &current_state))
-								needupdate = 1;
-	 						else if (current_state.chanpos == 16 &&
-	 							(cc[x].sigtype & ZT_SIG_CLEAR) && 
-	 							!(lc[current_state.spanno - 1].lineconfig &
-	 							(ZT_CONFIG_D4|ZT_CONFIG_ESF|ZT_CONFIG_B8ZS|ZT_CONFIG_CCS))) {
-	 								fprintf(stderr, 
-	 									"CAS signalling on span %u conflicts with %s on channel %u.\n",
-	 									current_state.spanno, 
-	 									sigtype_to_str(cc[x].sigtype), x);
-	 								exit(1);
-	 						}
-						}
-
-						if (!needupdate) {
-							master = current_state.channo >> 16;
-							
-							if (cc[x].sigtype != current_state.sigtype) {
-								needupdate++;
-								if (verbose > 1)
-									printf("Changing signalling on channel %d from %s to %s\n",
-									       cc[x].chan, sigtype_to_str(current_state.sigtype),
-									       sigtype_to_str(cc[x].sigtype));
-							}
-							
-							if ((cc[x].deflaw != ZT_LAW_DEFAULT) && (cc[x].deflaw != current_state.curlaw)) {
-								needupdate++;
-								if (verbose > 1)
-									printf("Changing law on channel %d from %s to %s\n",
-									       cc[x].chan, laws[current_state.curlaw],
-									       laws[cc[x].deflaw]);
-							}
-							
-							if (cc[x].master != master) {
-								needupdate++;
-								if (verbose > 1)
-									printf("Changing master of channel %d from %d to %d\n",
-									       cc[x].chan, master,
-									       cc[x].master);
-							}
-							
-							if (cc[x].idlebits != current_state.idlebits) {
-								needupdate++;
-								if (verbose > 1)
-									printf("Changing idle bits of channel %d from %d to %d\n",
-									       cc[x].chan, current_state.idlebits,
-									       cc[x].idlebits);
-							}
-						}
-
-						if (needupdate && ioctl(fd, ZT_CHANCONFIG, &cc[x])) {
-							fprintf(stderr, "ZT_CHANCONFIG failed on channel %d: %s (%d)\n", x, strerror(errno), errno);
-							if (errno == EINVAL) {
-								fprintf(stderr, "Did you forget that FXS interfaces are configured with FXO signalling\n"
-									"and that FXO interfaces use FXS signalling?\n");
-							}
-							close(fd);
-							exit(1);
-						}
-					}
-					for (x=0;x<numzones;x++) {
-						if (debug & DEBUG_APPLY) {
-							printf("Loading tone zone for %s\n", zonestoload[x]);
-							fflush(stdout);
-						}
-						if (tone_zone_register(fd, zonestoload[x]))
-							error("Unable to register tone zone '%s'\n", zonestoload[x]);
+				}
+				for (x=0;x<numdynamic;x++) {
+					if (ioctl(fd, ZT_DYNAMIC_CREATE, &zds[x])) {
+						fprintf(stderr, "Zaptel dynamic span creation failed: %s\n", strerror(errno));
+						close(fd);
+						exit(1);
 					}
+				}
+				for (x=1;x<ZT_MAX_CHANNELS;x++) {
+					struct zt_params current_state;
+					int master;
+					int needupdate = force;
+					
 					if (debug & DEBUG_APPLY) {
-						printf("Doing startup\n");
+						printf("Configuring device %d\n", x);
 						fflush(stdout);
 					}
-					if (deftonezone > -1) {
-						if (ioctl(fd, ZT_DEFAULTZONE, &deftonezone)) {
-							fprintf(stderr, "ZT_DEFAULTZONE failed: %s (%d)\n", strerror(errno), errno);
-							close(fd);
-							exit(1);
+					if (!cc[x].sigtype)
+						continue;
+					
+					if (!needupdate) {
+						memset(&current_state, 0, sizeof(current_state));
+						current_state.channo = cc[x].chan | ZT_GET_PARAMS_RETURN_MASTER;
+						if (ioctl(fd, ZT_GET_PARAMS, &current_state))
+							needupdate = 1;
+						else if (current_state.chanpos == 16 &&
+							(cc[x].sigtype & ZT_SIG_CLEAR) && 
+							!(lc[current_state.spanno - 1].lineconfig &
+							(ZT_CONFIG_D4|ZT_CONFIG_ESF|ZT_CONFIG_B8ZS|ZT_CONFIG_CCS))) {
+								fprintf(stderr, 
+									"CAS signalling on span %u conflicts with %s on channel %u.\n",
+									current_state.spanno, 
+									sigtype_to_str(cc[x].sigtype), x);
+								exit(1);
 						}
 					}
-					for (x=0;x<spans;x++) {
-						if (ioctl(fd, ZT_STARTUP, &lc[x].span)) {
-							fprintf(stderr, "Zaptel startup failed: %s\n", strerror(errno));
-							close(fd);
-							exit(1);
+					
+					if (!needupdate) {
+						master = current_state.channo >> 16;
+						
+						if (cc[x].sigtype != current_state.sigtype) {
+							needupdate++;
+							if (verbose > 1)
+								printf("Changing signalling on channel %d from %s to %s\n",
+								       cc[x].chan, sigtype_to_str(current_state.sigtype),
+								       sigtype_to_str(cc[x].sigtype));
 						}
+						
+						if ((cc[x].deflaw != ZT_LAW_DEFAULT) && (cc[x].deflaw != current_state.curlaw)) {
+							needupdate++;
+							if (verbose > 1)
+								printf("Changing law on channel %d from %s to %s\n",
+								       cc[x].chan, laws[current_state.curlaw],
+								       laws[cc[x].deflaw]);
+						}
+						
+						if (cc[x].master != master) {
+							needupdate++;
+							if (verbose > 1)
+								printf("Changing master of channel %d from %d to %d\n",
+								       cc[x].chan, master,
+								       cc[x].master);
+						}
+						
+						if (cc[x].idlebits != current_state.idlebits) {
+							needupdate++;
+							if (verbose > 1)
+								printf("Changing idle bits of channel %d from %d to %d\n",
+								       cc[x].chan, current_state.idlebits,
+								       cc[x].idlebits);
+						}
+					}
+					
+					if (needupdate && ioctl(fd, ZT_CHANCONFIG, &cc[x])) {
+						fprintf(stderr, "ZT_CHANCONFIG failed on channel %d: %s (%d)\n", x, strerror(errno), errno);
+						if (errno == EINVAL) {
+							fprintf(stderr, "Did you forget that FXS interfaces are configured with FXO signalling\n"
+								"and that FXO interfaces use FXS signalling?\n");
+						}
+						close(fd);
+						exit(1);
+					}
+				}
+				for (x=0;x<numzones;x++) {
+					if (debug & DEBUG_APPLY) {
+						printf("Loading tone zone for %s\n", zonestoload[x]);
+						fflush(stdout);
+					}
+					if (tone_zone_register(fd, zonestoload[x]))
+						error("Unable to register tone zone '%s'\n", zonestoload[x]);
+				}
+				if (debug & DEBUG_APPLY) {
+					printf("Doing startup\n");
+					fflush(stdout);
+				}
+				if (deftonezone > -1) {
+					if (ioctl(fd, ZT_DEFAULTZONE, &deftonezone)) {
+						fprintf(stderr, "ZT_DEFAULTZONE failed: %s (%d)\n", strerror(errno), errno);
+						close(fd);
+						exit(1);
+					}
+				}
+				for (x=0;x<spans;x++) {
+					if (ioctl(fd, ZT_STARTUP, &lc[x].span)) {
+						fprintf(stderr, "Zaptel startup failed: %s\n", strerror(errno));
+						close(fd);
+						exit(1);
 					}
 				}
 			}
