@@ -39,7 +39,7 @@
 #ifdef STANDALONE_ZAPATA
 #include "zaptel.h"
 #else
-#include <linux/zaptel.h>
+#include <zaptel/zaptel.h>
 #endif
 
 #define ETH_P_ZTDETH	0xd00d
@@ -56,6 +56,8 @@ static DEFINE_SPINLOCK(zlock);
 static spinlock_t zlock = SPIN_LOCK_UNLOCKED;
 #endif
 
+static struct sk_buff_head skbs;
+
 static struct ztdeth {
 	unsigned char addr[ETH_ALEN];
 	unsigned short subaddr; /* Network byte order */
@@ -175,7 +177,7 @@ static int ztdeth_transmit(void *pvt, unsigned char *msg, int msglen)
 			skb->dev = dev;
 			if (dev->hard_header)
 				dev->hard_header(skb, dev, ETH_P_ZTDETH, addr, dev->dev_addr, skb->len);
-			dev_queue_xmit(skb);
+			skb_queue_tail(&skbs, skb);
 		}
 	}
 	else
@@ -183,6 +185,18 @@ static int ztdeth_transmit(void *pvt, unsigned char *msg, int msglen)
 	return 0;
 }
 
+
+static int ztdeth_flush(void)
+{
+	struct sk_buff *skb;
+
+	/* Handle all transmissions now */
+	while ((skb = skb_dequeue(&skbs))) {
+		dev_queue_xmit(skb);
+	}
+	return 0;
+}
+
 static struct packet_type ztdeth_ptype = {
 	type: __constant_htons(ETH_P_ZTDETH),		/* Protocol */
 	dev: NULL,					/* Device (NULL = wildcard) */
@@ -270,6 +284,8 @@ static void ztdeth_destroy(void *pvt)
 		kfree(z);
 #ifndef LINUX26
 		MOD_DEC_USE_COUNT;
+#else
+		module_put(THIS_MODULE);
 #endif
 	}
 }
@@ -363,13 +379,16 @@ static void *ztdeth_create(struct zt_span *span, char *addr)
 			sprintf(src + strlen(src), "%02x:", z->dev->dev_addr[x]);
 		sprintf(src + strlen(src), "%02x", z->dev->dev_addr[5]);
 		printk("TDMoE: Added new interface for %s at %s (addr=%s, src=%s, subaddr=%d)\n", span->name, z->dev->name, addr, src, ntohs(z->subaddr));
-			
+
 		spin_lock_irqsave(&zlock, flags);
 		z->next = zdevs;
 		zdevs = z;
 		spin_unlock_irqrestore(&zlock, flags);
 #ifndef LINUX26
 		MOD_INC_USE_COUNT;
+#else
+		if(!try_module_get(THIS_MODULE))
+			printk("TDMoE: Unable to increment module use count\n");
 #endif
 	}
 	return z;
@@ -380,7 +399,8 @@ static struct zt_dynamic_driver ztd_eth = {
 	"Ethernet",
 	ztdeth_create,
 	ztdeth_destroy,
-	ztdeth_transmit
+	ztdeth_transmit,
+	ztdeth_flush
 };
 
 static struct notifier_block ztdeth_nblock = {
@@ -392,6 +412,9 @@ static int __init ztdeth_init(void)
 	dev_add_pack(&ztdeth_ptype);
 	register_netdevice_notifier(&ztdeth_nblock);
 	zt_dynamic_register(&ztd_eth);
+
+	skb_queue_head_init(&skbs);
+
 	return 0;
 }
 
