@@ -27,6 +27,7 @@ int main(int argc, char *argv[])
 	int fd;
 	int res;
 	int count=0;
+	int seconds;
 	int ms;
 	int curarg = 1;
 	int verbose=0;
@@ -41,16 +42,21 @@ int main(int argc, char *argv[])
 	while(curarg < argc) {
 		if (!strcasecmp(argv[curarg], "-v"))
 			verbose++;
+		if (!strcasecmp(argv[curarg], "-c") && argc > curarg)
+			seconds = atoi(argv[curarg + 1]);
 		curarg++;
 	}
 	printf("Opened pseudo zap interface, measuring accuracy...\n");
 	signal(SIGHUP, hup_handler);
 	signal(SIGINT, hup_handler);
+	signal(SIGALRM, hup_handler);
 	/* Flush input buffer */
 	for (count = 0;count < 4; count++)
 		res = read(fd, buf, sizeof(buf));
 	count = 0;
 	gettimeofday(&start, NULL);
+	if (seconds > 0)
+		alarm(seconds + 1);
 	for(;;) {
 		res = read(fd, buf, sizeof(buf));
 		if (res < 0) {
