@@ -42,8 +42,8 @@
 #include "zaptel.h"
 #include "tonezone.h"
 #else
-#include <linux/zaptel.h>
-#include <tonezone.h>
+#include <zaptel/zaptel.h>
+#include <zaptel/tonezone.h>
 #endif
 
 static int ctl = -1;
